/******************
* global variables
*******************/
let gIFrame; //the basket iframe
let gSession; //session object to manage EAIWS session
let gLoginHandler; //pCon.login handler

// setup initial value for the "Use pCon.login" checkbox
document.addEventListener("DOMContentLoaded", () => {
    document.getElementById("checkboxUseLogin").checked = PCON_LOGIN_CLIENT_ID != "" && window.location.protocol.startsWith("http");
});

/**
 * function to create and initialize the basket iframe
 */
async function startBasket() {

    gIFrame = document.getElementById("basketFrame"); //get basket iframe
    gSession = new egrWcf.eaiws.EaiwsSession(); //create sesssion object
    gLoginHandler = new LoginHandler(); //create pCon.login handler

    //check if login is enabled
    const tUseLogin = document.getElementById("checkboxUseLogin").checked;

    //loggin with pCon.login
    if (tUseLogin) {
        const tLoggedIn = await gLoginHandler.loginWithPopup(gSession, sendMessageToBasket);
        if (!tLoggedIn) {
            return;
        }
    }

    //hide intro
    document.getElementById("intro").style.display = "none";

    //register message handler to receive messages from basket
    window.addEventListener("message", function (pEvent) {
        //check if message is coming from basket
        if (pEvent.source !== gIFrame.contentWindow)
            return;

        //call our handler
        handleBasketMessage(pEvent.data);
    });

    //load pCon.basket in integration mode and make iframe visible
    gIFrame.src = BASKET_URL;
    gIFrame.style.display = "block";
}

/**
 * This function handles every message coming from pCon.basket
 */
function handleBasketMessage(pMessage) {
    console.dir(pMessage);

    //basket requests its configuration => prepare configuration and send it to basket
    if (pMessage.type === "wbkHost.getConfiguration") {
        console.log("Basket version: " + pMessage.parameter.appVersion);
        console.log("Api version: " + pMessage.parameter.messageApiVersion.major + "." + pMessage.parameter.messageApiVersion.minor );
        prepareAndSendConfiguration();
    } else

    //basket requests the article data for a custom article
    if (pMessage.type === "wbkHost.getCustomArticle") {
        handleCustomArticleMessage(pMessage.parameter, sendMessageToBasket);
    } else

    //basket requests the custom articles for our custom catalog
    if (pMessage.type === "wbkHost.getCustomCatalogArticles") {
        handleCustomCatalogArticlesMessage(pMessage.parameter, sendMessageToBasket);
    } else

    //project title changed
    if (pMessage.type === "wbkHost.projectTitleChanged") {
        //reset project title to "unnamed" if empty
        if (pMessage.parameter === "") {
            sendMessageToBasket({type: "wbk.setProjectTitle", parameter: "unnamed" });
        }
    } else

    //user clicked the save project button
    if (pMessage.type === "wbkHost.saveProject") {
        saveProject();
    } else

    //user is finished with editing
    if (pMessage.type === "wbkHost.done") {
        showItemList();
    }
}

/**
 * This function is used to send messages to the basket
 */
function sendMessageToBasket(pMessage) {
    gIFrame.contentWindow.postMessage(pMessage, "*");
}

/**
 * This function prepares the configuration of pCon.basket including the EAIWS session and sends it to the basket.
 */
async function prepareAndSendConfiguration() {
    //option1: create EAIWS session using gatekeeper. see gatekeeper documentation: https://eaiws-server.pcon-solutions.com/doc/v3
    await getSessionFromGatekeeper(gSession, GATEKEEPER_ID, gLoginHandler.accessToken);

    //option2: create EAIWS session using eaiws baseurl and startup (not supported in pCon.cloud)
    //await gSession.open("https://www.example-eaiws.com", {startup: example-startup"});

    //prepare/intialize the session
    const tApplySessionDefault = await preareEaiwsSession();

    //build the configuration message and send it to pCon.basket
    let tMessage = {
        type: "wbk.configuration",
        parameter: {
            application: {
                theme: { //use custom application colors
                    primaryColor: "#00A000",
                    appBarColor: "#505050"
                },
                //featureSet: "Configurator", //enable configurator mode
                showUser: true,
                showProjectSaveButton: true //see handling of `wbkHost.saveProject` message
            },
            user: {
                /* The pCon.login access token should be provided here, to activate all application features.
                 * e.g. accessing protected PIM article information or to access user/organization preferences
                 * Note: The access token usually expires after 1 hour. Its important to request a new token before that happens and send it
                 * to the basket using the `wbk.updateUserAccessToken` message.
                 */
                accessToken: gLoginHandler.accessToken,
                name: "user@example.com",
                fullName: "Example User"
            },
            eaiws: {
                baseUrl: gSession.baseUrl,
                sessionId: gSession.session.sessionId,
                keepAliveInterval: 0, //disable keep alive, its already handled on our side
            },
            calculation: {
                pricingProcedure: PRICING_PROCEDURE_NAME //use different pricing procedure (optional)
            },
            project: {
                title: "test project",
                titleEditable: true,
                applySessionDefaults: tApplySessionDefault, //start new project with defaults
                openCatalogForEmptyProjects: true //start in catalog
            },
            report: {
                templates: [REPORT_TEMPLATE],
                defaultTemplate: REPORT_TEMPLATE
            },
            catalog: {
                customCatalogs: getCustomCatalogConfiguration()
            }
        }
    }
    sendMessageToBasket(tMessage);
}

/**
 * This function is used to initailize the EAIWS session.
 */
async function preareEaiwsSession() {
    //get selected file
    let tInputField = document.getElementById("fileInput");
    let tSelectedFile = tInputField.files[0];
    let tApplySessionDefaults = true;

    if (tSelectedFile != null && (tSelectedFile.name).toLowerCase().endsWith(".obk")) {
        //OBK selected -> load OBK
        await loadOBK(tSelectedFile);
        tApplySessionDefaults = false; //do not apply session defaults, keep settings stored in obk
    } else if (tSelectedFile != null && (tSelectedFile.name).toLowerCase().endsWith(".obx")) {
        //OBX selected -> load OBX
        await loadOBX(tSelectedFile);
    } else {
        //no file -> create new/empty project with prefilled header data
        await setExampleProjectData(gSession);
        await setExampleCustomerAddress(gSession);
    }

    //setup our price conditions
    await setupPriceConditions(gSession);
    return (tApplySessionDefaults);
}

/**
 * This function loads and OBK file.
 */
async function loadOBK(pFile) {
    //get upload URL from EAIWS
    const tUrl = await gSession.session.getUploadURL("Project");

    //upload file (using PUT request)
    await gSession.uploadFileToUrl(tUrl, pFile);

    //load session from uploaded obk
    await gSession.session.loadSession(tUrl);
}

/**
 * This function loads and OBX file.
 */
async function loadOBX(pFile) {
    //get upload URL from EAIWS
    const tUrl = await gSession.session.getUploadURL("CutBuffer");

    //upload file (using PUT request)
    await gSession.uploadFileToUrl(tUrl, pFile);

    //insert obx into the session
    await gSession.basket.paste(null, null, tUrl);
}

/**
 * This function is called when the user clicks the save button in pCon.basket UI and will save an OBK file.
 */
async function saveProject() {
    //save OBK
    await saveOBK();

    //notify basket that saving is done
    let tMessage = {
        type: "wbk.saveProjectDone"
    }
    sendMessageToBasket(tMessage);
}

/**
 * This function saves the whole project/session as an OBK file.
 */
async function saveOBK() {
    //get OBK url and download it
    const tUrl = await gSession.session.saveSession(null);
    window.location.href = tUrl;
}

/**
 * This function saves all items as an OBX file.
 */
async function saveOBX() {
    //get all basket items
    const tItems = await gSession.basket.getAllItems();

    //get url of OBX and download it
    const tItemIdArray = tItems.map((item) => item.itemId);
    const tUrl = await gSession.basket.copy(tItemIdArray);
    window.location.href = tUrl;
}

/**
 * This function generates and save a pdf report.
 */
async function savePDF() {
    //use EAIWS reporter plugin to generate a PDF
    const tResult = await egrWcf.utils.async.ajax(
        "POST",
        egrWcf.utils.string.joinPath(gSession.baseUrl, `/EAIWS/plugins/Reporter/template/${REPORT_TEMPLATE}/v1/generate`),
        {
            sessionId: gSession.sessionId,
            calculationScheme: PRICING_PROCEDURE_NAME,
            preferredImageColumn: COLUMN_ITEM_IMAGE,
            externalRefColumn: COLUMN_EXT_ITEM_NUMBER
        },
        { dataType: "json" }
    );
    window.location.href = tResult.url;
}

/**
 * Closes the current session and restarts the example.
 */
async function restart() {
    //close the session
    await gSession.close();

    //reload
    window.location.reload();
}

/**
 * This function collects all items and displays them in the item list.
 */
async function showItemList() {
    //close/hide basket first
    gIFrame.src = "";
    gIFrame.style.display = "none";

    //show output
    document.getElementById("output").style.display = "block";

    /*
    Get all items
    Because the `positionNumber` field is only supported for view items we are requesting all
    view items of the standard view. If `positionNumber` is not required  we could directly request
    basket items instead (leave `viewId` undefined).
    */
    const tOptions = new egrWcf.eaiws.basket.ItemSelectionOptions();
    tOptions.viewId = "5726009a-756d-11d6-9c21-00e029099a4b" //id of the standard view

    const tItems = await gSession.basket.getAllItems(undefined, tOptions);
    for (var i = 0; i < tItems.length; ++i) {
        showItem(tItems[i]);
    }
}

/**
 * This function collects all information about an item and displays it.
 */
async function fetchAndShowItemInfo(pBasketItemId, pItemNumber) {
    //get item properties
    const tGetPropsOptions = new egrWcf.eaiws.basket.GetItemPropertiesOptions();
    tGetPropsOptions.separateCurrencies = true; //enable separate purchase and sales currency support
    const tItemProps = await gSession.basket.getItemProperties([pBasketItemId], tGetPropsOptions);

    //get item calculation
    const tCalculations = await gSession.basket.getPriceCalculationSheets([pBasketItemId], PRICING_PROCEDURE_NAME);

    //get default item image from the default image column
    const tGetFieldsOptions = new egrWcf.eaiws.basket.GetItemFieldsOptions();
    tGetFieldsOptions.useDefault = true;
    tGetFieldsOptions.resolveURI = true;
    tGetFieldsOptions.generateImages = true;
    const tFields = await gSession.basket.getItemFields([pBasketItemId], [COLUMN_ITEM_IMAGE], tGetFieldsOptions);
    let tImageUrl = undefined;
    if (tFields.length > 0) {
        tImageUrl = tFields[0].data;
    }

    showItemInfo(pItemNumber, tItemProps[0], tCalculations[0].sheet, tImageUrl);
}


/******************
* Helper functions
*******************/

/**
 * This function will clear the value of the file input.
 */
function clearFileInput() {
    document.getElementById("fileInput").value = null;
}

/**
 * Helper to show an item to the item list.
 */
function showItem(pItem) {
    const addCell = function (pValue) {
        const tCell = document.createElement("td");
        tCell.innerText = pValue;
        tRow.appendChild(tCell);
    }

    const tTable = document.getElementById("itemList");
    const tRow = document.createElement("tr");

    addCell(pItem.positionNumber);
    addCell(pItem.label);
    addCell(pItem.itemType);

    //because we requested view items the field `itemId` represents the id of the view item
    //we have to use the `basketItemIds`in this case
    const tBasketItemId = pItem.basketItemIds[0];
    addCell(tBasketItemId);

    //add button cell
    const tCell = document.createElement("td");
    const tButton = document.createElement("button");
    tButton.innerText = "i";
    tButton.style.minWidth = "24px";
    tButton.onclick = () => fetchAndShowItemInfo(tBasketItemId, pItem.positionNumber);
    tCell.appendChild(tButton);
    tRow.appendChild(tCell);

    tTable.appendChild(tRow);
}

/**
 * Helper to show additional information for an item.
 */
async function showItemInfo(pItemNumber, pItemProps, pCalculation, pImageUrl) {
    const addInfo = function (pName, pValue, pRed) {
        const tInfo = document.createElement("div");
        const tInfoName = document.createElement("span");
        const tInfoValue = document.createElement("span");
        tInfo.style.paddingBottom = "6px";
        if (pRed === true) {
            tInfo.style.color = "red";
        }
        tInfoName.style.fontWeight = 600;
        tInfoName.innerText = pName + ": ";
        tInfoValue.innerText = pValue;
        tInfo.appendChild(tInfoName);
        tInfo.appendChild(tInfoValue);
        tItemInfoContent.appendChild(tInfo);
    }

    const formatPrice = function (pValue, pCurrency) {
        if (pValue == null || pCurrency == null)
            return ("");
        return (egrWcf.utils.number.toLocaleCurrencyString(pValue, pCurrency, "en-US"));
    }

    //clear item info
    const tItemInfoContent = document.getElementById("itemInfoContent");
    tItemInfoContent.innerHTML = "";

    addInfo("Item Number", pItemNumber);
    addInfo("Type", pItemProps.itemType);
    addInfo("Item ID", pItemProps.itemId);
    if (pItemProps.article != null) {
        addInfo("Manufacturer ID", pItemProps.article.manufacturerId);
        addInfo("Manufacturer Name", pItemProps.article.manufacturerName);
        addInfo("Series ID", pItemProps.article.seriesId);
        addInfo("Series Name", pItemProps.article.seriesName);
        if (pItemProps.article.useFinalArticleNumber === true) {
            addInfo("Article Number", pItemProps.article.finalArticleNumber);
        } else {
            addInfo("Article Number", pItemProps.article.baseArticleNumber);
        }
        addInfo("Variant Code", pItemProps.article.variantCode);
        addInfo("Short Text", pItemProps.article.shortText);
        addInfo("Long Text", pItemProps.article.longText);
        addInfo("Feature Text", pItemProps.article.featureText);
        if (pItemProps.article.inconsistencyReason != null && pItemProps.article.inconsistencyReason.length > 0) {
            addInfo("Inconsistency Reason", pItemProps.article.inconsistencyReason.join("\n"), true);
        }
        addInfo("Base Purchase Price", formatPrice(pItemProps.article.purchasePrice, pItemProps.article.purchaseCurrency));
        addInfo("Base Sales Price", formatPrice(pItemProps.article.salesPrice, pItemProps.article.salesCurrency));
        addInfo("Quantity", pItemProps.article.quantity);
    } else {
        addInfo("Label", pItemProps.label);
    }

    //find net value in calculation and print it
    let tNetValueCalcLine = undefined;
    for (let i = 0; i < pCalculation.lines.length; ++i) {
        if (pCalculation.lines[i].level === NETVALUE_CALCLINE_LEVEL) {
            tNetValueCalcLine = pCalculation.lines[i];
            break;
        }
    }
    if (tNetValueCalcLine != null) {
        if (tNetValueCalcLine.amount != null) {
            addInfo("Unit Net Price", formatPrice(tNetValueCalcLine.amount.value, tNetValueCalcLine.amount.currency));
        }
        if (tNetValueCalcLine.value != null) {
            addInfo("Total Net Price", formatPrice(tNetValueCalcLine.value.value, tNetValueCalcLine.value.currency));
        }
    }

    //item image
    if (pImageUrl != null && pImageUrl != "") {
        const tImage = document.createElement("img");
        tImage.src = pImageUrl;
        tImage.style.maxWidth = "200px";
        tItemInfoContent.appendChild(tImage);
    }
}
