class LoginHandler {
    static POPUP_URL = "login-popup.html";
    accessToken = undefined;

    constructor() {
        //create an instance of PConLoginClient
        this.#loginClient = new PConLogin.PConLoginClient({
            client_id: PCON_LOGIN_CLIENT_ID,
            scope: PCON_LOGIN_SCOPE,
            redirect_uri: window.location.origin + window.location.pathname + LoginHandler.POPUP_URL,
        });
    }

    /**
     * Function which starts the pCon.login process using a popup.
     */
    async loginWithPopup(pSession, pSendMessageCallback) {
        //check if client id is configured
        if (PCON_LOGIN_CLIENT_ID == null || PCON_LOGIN_CLIENT_ID == "") {
            alert('PCON_LOGIN_CLIENT_ID not defined.\nPlease configure a valid pCon.login client id in the "config.js" file.');
            return;
        }

        //check if example is hosted on a webserver (no "file://")
        if (!window.location.protocol.startsWith("http")) {
            alert("Please use a webserver to run this example. pCon.login will not be able to call the redirect url otherwise.");
            return;
        }

        //show the login popup and wait until its finished
        const tLoggedIn = await this.#showLoginPopup();
        if (!tLoggedIn) {
            return (false);
        }

        //get access token
        await this.#loginClient.handlePopupFinished();
        this.accessToken = await this.#loginClient.getAccessToken();

        //keep license live (IMPORTANT: check error handling in this function)
        this.#startLicenseKeepAlive(pSession);

        //make sure the basket ui always has a valid access token
        this.#startBasketAccessTokenUpdate(pSendMessageCallback);

        return (true);
    }

    //// private ///////////////////////////////////////////////////////////////////////////////////

    #loginClient = undefined;

    /** Makes sure the license will not expire. */
    #startLicenseKeepAlive(pSession) {
        //trigger license update every 30 seconds to keep the license alive
        setInterval(async () => {
            try {
                this.accessToken = await this.#loginClient.getAccessToken(); //make sure a valid access token is used
                const tMessage = new egrWcf.eaiws.session.ClientMessage();
                tMessage.target = "plugin:eaiws::plugin::session_guard";
                tMessage.messageId = "updateToken";
                tMessage.data = [
                    new egrWcf.eaiws.session.StringPair("accessToken", this.accessToken),
                    new egrWcf.eaiws.session.StringPair("licenseType", "user")
                ];

                const tResult = await pSession.session.sendMessage([tMessage]);

                //check if token update was successfull
                let tUpdateFailed = true;
                if (tResult.sessionAlive) {
                    tUpdateFailed = tResult.messages.length > 0 && tResult.messages[0].statusCode !== "OK";
                }
                if (tUpdateFailed) {
                    throw new Error("Failed to update token:\n" + JSON.stringify(tResult, undefined, 2));
                }
            } catch (pError) {
                /*
                IMPORTANT: You have to handle this error.
                The license keep alive failed. That means the current EAIWS session is no longer licensed.
                The session will be closed automatically after 10 minutes without a license.
                To prevent data loss you should notify the user that he should save his work and restart the basket.
                */
                alert("License keep alive failed:\n" + pError);
            }
        }, 30000);
    }

    /** Ensures that the basket ui always has a valid access token */
    #startBasketAccessTokenUpdate(pSendMessageCallback) {
        /*
        Access tokens are valid for one hour. To make sure that the basket UI is using a valid
        token we send a new token every 30 minuten to the basket.
        */
        setInterval(async () => {
            this.accessToken = await this.#loginClient.getAccessToken(true); //request a new token using the "force" option
            const tMessage = {
                type: "wbk.updateUserAccessToken",
                parameter: this.accessToken
            }
            pSendMessageCallback(tMessage)
        }, 1800000);
    }

    /** Creates and shows the login popup. Returns false login was aborted by user. */
    async #showLoginPopup() {
        let tLoggedIn = false;
        let tPopup;
        let popupPromiseResolve;

        //Message handler which handles the "login-popup-finished" message from the popup
        const onMessage = (pEvent) => {
            if (pEvent.source === tPopup && pEvent.data === "login-popup-finished") {
                tLoggedIn = true;
                popupPromiseResolve();
                tPopup.close();
            }
        };

        //Timer which checks if the popup was closed. This is necassary because the user may close the popup without loggin in.
        const tCloseTimer = setInterval(() => {
            if (tPopup != null && tPopup.closed) {
                clearInterval(tCloseTimer);
                popupPromiseResolve();
            }
        }, 500);

        //register temporary message handler
        window.addEventListener("message", onMessage);

        //create the popup window and wait untel the promise resolves
        await new Promise((resolve, reject) => {
            popupPromiseResolve = resolve;
            tPopup = window.open(LoginHandler.POPUP_URL, "_blank", this.#getPopupFeatures());
        });

        //remove the temporary message handler
        window.removeEventListener("message", onMessage);
        return (tLoggedIn);
    }

    /** Returns the feature string for the login popup window */
    #getPopupFeatures() {
        //position the popup in the center of the screen
        const tWidth = 500;
        const tHeight = 600;
        const tLeft = Math.round(window.screen.width / 2 - tWidth / 2);
        const tTop = Math.round(window.screen.height / 2 - tHeight / 2);
        return (`popup,width=${tWidth},height=${tHeight},left=${tLeft},top=${tTop}`);
    }
}