import { EaiwsSession } from "@egr/wcf/modules/eaiws";
import * as basket from "@egr/wcf/modules/eaiws/basket";
import * as project from "@egr/wcf/modules/eaiws/project";

/**
 * Provides commonly used localized resources.
 * The current language of the application will be used as resource language.
 */
export interface CommonResources
{
    readonly buttonOk: string;
    readonly buttonCancel: string;
    readonly buttonAbort: string;
    readonly buttonYes: string;
    readonly buttonNo: string;
    readonly buttonDone: string;
    readonly buttonApply: string;
    readonly buttonContinue: string;

    readonly labelError: string;
    readonly labelUpload: string;
    readonly labelDownload: string;
    readonly labelShare: string;
    readonly labelLogin: string;
    readonly labelLogout: string;
    readonly labelCut: string;
    readonly labelCopy: string;
    readonly labelPaste: string;
    readonly labelOpen: string;
    readonly labelClose: string;
    readonly labelSave: string;
    readonly labelSaveAsDefault: string;
    readonly labelDeleteDefaults: string;
    readonly labelInconsistenArticles: string;
    readonly labelUnsupportedFile: string;
    readonly labelUnsupportedImage: string;

    readonly errorStartSessionFailed: string;
    readonly errorOpenProjectFailed: string;
    readonly errorImportFailed: string;
    readonly errorExportFailed: string;
    readonly errorInsertArticleFailed: string;
    readonly errorUnexpected: string;
    readonly errorInvalidUsernameOrPassword: string;

    /** @deprecated */
    readonly confirmationInconsistenArticles: string;

    readonly confirmationInconsistentArticles: string;

    readonly placeholderUnnamed: string;

    readonly itemDataManufacturer: string;
    readonly itemDataManufacturerId: string;
    readonly itemDataSeries: string;
    readonly itemDataSeriesId: string;
    readonly itemDataArticleNumber: string;
    readonly itemDataPriceDate: string;
    readonly itemDataPurchasePrice: string;
    readonly itemDataSalesPrice: string;
    readonly itemDataPackagingNetWeight: string;
    readonly itemDataPackagingVolume: string;
    readonly itemDataShortText: string;
    readonly itemDataLongText: string;
    readonly itemDataFeatureText: string;
    readonly itemDataUserShortText: string;
    readonly itemDataUserLongText: string;
    readonly itemDataUserFeatureText: string;

    readonly itemInactiveStateAlternative: string;
    readonly itemInactiveStateOptional: string;

    readonly calcDataItemQuantity: string;
    readonly calcDataCondDescription: string;
    readonly calcDataCondType: string;
    readonly calcDataCondAmount: string;
    readonly calcDataCondValue: string;

    readonly projectDataDate: string;
    readonly projectDataDescription: string;
    readonly projectDataExtRefNumber: string;
    readonly projectDataExtRefText: string;
    readonly projectDataKeywords: string;
    readonly projectDataName: string;
    readonly projectDataPersonInCharge: string;
    readonly projectDataPriceList: string;
    readonly projectDataProjectGroup: string;
    readonly projectDataProjectNumber: string;
    readonly projectDataStatus: string;
    readonly projectDataTermsAndConditions: string;

    readonly projectStatusUndefined: string;
    readonly projectStatusInProgress: string;
    readonly projectStatusOffered: string;
    readonly projectStatusNotCommissioned: string;
    readonly projectStatusApprovalRequired: string;
    readonly projectStatusWaitingForApproval: string;
    readonly projectStatusApprovalDenied: string;
    readonly projectStatusApproved: string;
    readonly projectStatusOrdered: string;

    readonly quoteDataDate: string;
    readonly quoteDataLanguage: string;
    readonly quoteDataQuoteNumber: string;
    readonly quoteDataValidToDate: string;

    readonly contactDataCity: string;
    readonly contactDataCompany: string;
    readonly contactDataCountry: string;
    readonly contactDataCustomerNumber: string;
    readonly contactDataDepartment: string;
    readonly contactDataDistrict: string;
    readonly contactDataEmail: string;
    readonly contactDataFax: string;
    readonly contactDataFirstName: string;
    readonly contactDataGeneralAgreementNumber: string;
    readonly contactDataConditionNumber: string;
    readonly contactDataLastName: string;
    readonly contactDataMobile: string;
    readonly contactDataPhone: string;
    readonly contactDataPOBox: string;
    readonly contactDataPOBoxPostcode: string;
    readonly contactDataPosition: string;
    readonly contactDataPostcode: string;
    readonly contactDataRegion: string;
    readonly contactDataRemarks: string;
    readonly contactDataSalutation: string;
    readonly contactDataSalutationMr: string;
    readonly contactDataSalutationMrs: string;
    readonly contactDataStreet: string;
    readonly contactDataWebsite: string;
    readonly contactDataMainAddress: string;
    readonly contactDataShippingAddress: string;
    readonly contactDataBillingAddress: string;

    /**
     * Helper function to retrieve the countries from an eaiws session.
     * This function caches the server response to speed up further calls.
     */
    getCountries(pSession: EaiwsSession): Promise<Array<project.CountryData>>;

    /**
     * Helper function to retrieve the country subdivisions from an eaiws session.
     * This function caches the server response to speed up further calls.
     */
    getCountrySubdivisions(pSession: EaiwsSession, pCountryCode: string): Promise<Array<project.SubdivisionData>>;

    /**
     * Helper function to retrieve a localized country name from an eaiws session.
     * This function caches the server response to speed up further calls.
     */
    getCountryName(pSession: EaiwsSession, pCountryCode: string): Promise<string | null>;

    /**
    * Helper function to retrieve a localized region/subdivision name from an eaiws session.
    * E.g. country = US, region = OH will return "Ohio"
    * This function caches the server response to speed up further calls.
    */
    getRegionName(pSession: EaiwsSession, pCountryCode: string, pRegionCode: string): Promise<string | null>;

    /** Returns a localized name for the given currency code. */
    getCurrencyName(pCurrencyCode: string): string | null;

    /** Returns an array with narrow day names (e.g. S, M, T) starting with sunday. */
    getDayNamesNarrow(): Array<string>;

    /** Returns an array with day names starting with sunday. */
    getDayNames(): Array<string>;

    /** Returns an array with abbreviated month names (e.g. jan, feb) starting with january. */
    getMonthNamesAbbreviated(): Array<string>;

    /** Returns an array with month names starting with january. */
    getMonthNames(): Array<string>;

    /** Returns a set of strings which can be used for calendar or date picker components. */
    getCalendarStrings(): CalendarStrings;

    /**
     * Helper function to retrieve a localized name for the given project group.
     * If pPreferFullName is true, the full name of the group will be returned if available.
     */
    getEaiwsProjectGroupName(pProjectGroup: project.ProjectGroup, pPreferFullName: boolean): string;

    /**
     * Helper function to retrieve a localized name for the given price list.
     */
    getEaiwsPriceListName(pPriceList: project.PriceList): string;

    /**
     * Returns a text based on the current item status (e.g. inconsistency) which can be show as a warning to the user.
     */
    getBasketItemWarningText(pItemProps: basket.ItemProperties | null | undefined, pItemCalculation: basket.CalculationSheetInfo | null | undefined): string | null;

    /**
     * Returns an error text for the given calculation.
     */
    getCalculationErrorText(pCalculation: basket.CalculationSheetInfo | null | undefined, pIncludeGroupErrors: boolean): string | null;

    /**
     * Returns a text to inform the user that a file is not supopoted because of its file foramt.
     * @param pSupportedFormats List of supported file formats which will included in the text e.g. `jpg, png`
     */
    getUnsupportedFileFormatText(pSupportedFormats: string): string;

    /**
     * Returns a text for the given shortcut key e.g. `(Ctrl+C)`
     * Keys have to be specified according to the specification of `KeyboardEvent.key`.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key
     * @see https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key/Key_Values
     */
    getKeyboardShortcutText(pKey: string, pOptions?: KeyboardShortcutTextOptions): string;
}

export interface KeyboardShortcutTextOptions
{
    /** Optional first modfier key (e.g. `Control` or `Shift`) */
    modifier1?: string;
    /** Optional second modfier key (e.g. `Control` or `Shift`) */
    modifier2?: string;
    /**
     * If true the `Control` key will be replaced by the `Cmd` key on Apple operating systems.
     * @defaultValue true
     */
    useCommandOnAppleOS?: boolean;
}

export interface CalendarStrings
{
    months: string[];
    shortMonths: string[];
    days: string[];
    shortDays: string[];
    goToToday: string;
}