import { ProjectInfo } from "@egr/wcf/modules/bsk";
import { EaiwsSession } from "@egr/wcf/modules/eaiws";

import * as React from "react";
import { MenuAction } from "./Action";
import { PluginComponent } from "./PluginComponent";
import { Activity, ActivityHost } from "./plugins/Activity";
import { ProjectEditor, ProjectEditorOptions } from "./plugins/ProjectEditor";

/** Factory for reusable components. */
export interface ComponentFactory
{
    /**
     * React component which can be used to visualize a loading state.
     * If the isLoading property is true a loading animation will be shown. The user will not be able to interact
     * with the child components of the panel while loading (requires react).
     */
    loadingPanel(): typeof LoadingPanel;

    /**
     * React component which can be used to pick a list of languages (requires react).
     */
    dataLanguagePicker(): typeof DataLanguagePicker;

    /**
     * Creates a progress overlay which can be used to visualize a progress.
     * If the progress is no longer used, dispose() has to be called to
     * clean-up all resources.
     */
    createProgressOverlay(pOptions: ProgressOverlayOptions): ProgressOverlay;

    /**
     * Creates a new component container which can be used to display custom components at
     * application level (e.g. dialogs). If the container is no longer used, dispose() has to be called to
     * clean-up all resources.
     * @param pClassName Optional CSS class name for the container.
     */
    createRootComponentContainer(pClassName?: string): ComponentContainer;

    /**
     * Creates the default project editor activity.
     */
    createProjectEditorActivity(pHost: ActivityHost, pOptions?: ProjectEditorActivityOptions): Promise<Activity>;

    /**
     * Helper function to create a react based PluginComponent.
     */
    createReactComponent<T extends PluginComponent>(pContainer: HTMLElement, pCallback: CreateReactComponentCallback<T>): Promise<T>;
}


/** Container for custom components. */
export interface ComponentContainer
{
    /** HTML element which should be used as the root for the component. */
    readonly element: HTMLElement;

    /** Has to be called if the container is no longer in use. */
    dispose(): void;
}


/**
 * Options for `ComponentFactory.createProjectEditorActivity()`
 */
export interface ProjectEditorActivityOptions extends Omit<ProjectEditorOptions,
    "container" | "eaiwsSession" | "projectInfo">
{
    /** If true the project name will be used as the title for the activity. (default: true) */
    useProjectNameAsTitle?: boolean;

    /** If true the eaiws session will be closed if the activity gets disposed. (default: true) */
    closeSessionOnDispose?: boolean;

    /**
    * Can be used to create a custom eaiws session. If not implemented a default session will be created.
    * Null should be returned if the start of the project editor should be aborted.
    */
    onCreateSession?: () => Promise<EaiwsSession | null>;

    /**
    * Can be used to prepare the eaiws session before starting the project editor.
    * False should be returned if the start of the project editor should be aborted.
    * E.g. Can be used to load an existing project.
    */
    onPrepareSession?: (pSession: EaiwsSession) => Promise<boolean>;

    /**
    * Will be called before creating the project editor to define a pre-created `ProjectInfo` instance.
    */
    onGetProjectInfo?: (pSession: EaiwsSession) => ProjectInfo | null;

    /**
    * Will be called directly after the project editor was started.
    */
    onPrepareProjectEditor?: (pProjectEditor: ProjectEditor) => Promise<void>;

    /**
     * Will be called everytime the main menu gets opened.
     * Can be used to add new actions or override the project editor actions.
     * Note: use pProjectEditor.getMainMenuActions() to get the default actions of the editor.
     */
    onGetMainMenuActions?: (pProjectEditor: ProjectEditor | null) => ReadonlyArray<MenuAction> | null;

    /**
     * Can be implemented to make preparations before the activity gets stopped and disposed.
     * If `pCancelable` is true and the promise resolves with false, the activity stop will be canceled.
     * Otherwise true should be returned.
    */
    onPrepareActivityStop?: (pProjectEditor: ProjectEditor | null, pCancelable: boolean) => Promise<boolean>;
}


export interface ProgressOverlayOptions
{
    /* Title of the progress */
    title: string;

    /** If defined an abort button will be displayed. */
    onAbort?: () => void;
}


export interface ProgressOverlay
{
    /** Current progress [0,100]. If null, an indeterminate progress will be shown. */
    progress: number | null;

    /** Custom progress text to be shown. */
    progressText: string | undefined;

    /** If true the abort button will be disabled. */
    disableAbort: boolean;

    /** Has to be called if the progress is no longer in use. */
    dispose(): void;
}


interface BaseLoadingPanelProps extends React.ClassAttributes<LoadingPanel>
{
    /** if true the loading panel will show its loading animation */
    isLoading?: boolean;

    /** current progress [0,100] */
    progress?: number;

    /** If true the abort button will be disabled. (default: false) */
    disableAbort?: boolean;

    /** Additional css classes for the loading oerlay. */
    overlayClassName?: string;

    /** If defined an abort button will be displayed. */
    onAbort?: () => void;

    /** reference to the DOM div element of the loading panel */
    divRef?: React.Ref<HTMLDivElement>;
}
type DivProps = React.HTMLAttributes<HTMLDivElement>;

export type LoadingPanelProps = DivProps & BaseLoadingPanelProps;
export abstract class LoadingPanel extends React.Component<LoadingPanelProps> { }


export interface DataLanguagePickerProps
{
    /** Custom label for the picker */
    label?: string;

    /** Current value of the picker (array of language ids) */
    value?: Array<string>;

    /** Optional information text to show for the picker. */
    infoText?: string;

    /** Set to true if the picker shoulb be read only. */
    readonly?: boolean;

    /** Change handler for the picker. */
    onChange?: (pNewValue: Array<string>) => void;
}
export abstract class DataLanguagePicker extends React.Component<DataLanguagePickerProps> { }

/**
 * Callback for `ComponentFactory.createReactComponent()`.
 * @param pSetRef Ref callback for the react component.
 * @param pDispose Has to be called if `PluginComponent.dipose()` was called.
 */
export type CreateReactComponentCallback<T extends PluginComponent> = (
    pSetRef: (pComponent: T | null) => void,
    pDispose: () => void
) => JSX.Element;

