import * as catalog from "@egr/wcf/modules/eaiws/catalog";

import { ListenableEvent } from "@egr/wcf/modules/utils";
import { Plugin } from "../Plugin";
import { CustomArticle } from "./Catalog";

/**
 * Available sub categories for `CatalogExtension` plugins
 */
export type ExtensionSubCategory = "CatalogItemFilter" | "CustomCatalog";

/**
 * Additional options for `CatalogItemFilterExtension.filterCatalogItems()`.
 */
export interface FilterCatalogItemsOptions
{
    /**
     * Defines the context for the filter operation.
     * - `default`: default context e.g. the user is browsing the catalog
     * - `search`:  filter function was called to filter a search result
     */
    context: "default" | "search";
}

/**
 * Extension for the catalog which is used to filter the displayed catalog items.
 * - Plugin category = `CatalogExtension`
 * - Plugin sub category = `CatalogItemFilter`
 */
export interface CatalogItemFilterExtension extends Plugin
{
    /**
     * Optional list of catalog item resource types which can be used to request additional
     * resources that are necessary for filtering the catalog items.
     */
    readonly requiredResourceTypes?: Array<string>;

    filterCatalogItems(pItems: ReadonlyArray<catalog.CatalogItem>, pOptions: FilterCatalogItemsOptions): Array<catalog.CatalogItem>;
}

/**
 * Extension for the catalog which is used to provide custom catalogs.
 * - Plugin category = `CatalogExtension`
 * - Plugin sub category = `CustomCatalog`
 */
export interface CustomCatalogExtension extends Plugin
{
    /** Event wich should be triggered if the catalogs returned by `queryCatalogs` have changed. */
    eventCatalogsChanged?: ListenableEvent<void, CustomCatalogExtension>;

    /** Query a list of custom catalogs. */
    queryCatalogs(pQuery: CatalogQuery): Promise<Array<CustomCatalog>>;

    /** Query a list of articles. Note: Will not be called for external catalogs. */
    queryCatalogArticles(pQuery: CatalogArticlesQuery): Promise<Array<CustomCatalogArticle>>;

    /** Query detailed information of a single article. Note: Will not be called for external catalogs. */
    queryArticle(pQuery: ArticleQuery): Promise<CustomArticle>;
}

export interface CustomItem
{
    id: string;
    label: string;
    imageUrl?: string;
}

export interface CustomCatalog extends CustomItem
{
    /** If provided the catalog will be handled as an external catalog. It will be embedded in the ui using the `externalUrl`. */
    externalUrl?: string;
}

export interface CustomCatalogArticle extends CustomItem
{
    catalogId: string;
    articleNumber?: string;
    manufacturerId?: string;
    seriesId?: string;
}

export interface BaseQuery
{
    /** The requested language for the catalog/article data in order of priority. */
    language: Array<string>;
}

export interface CatalogQuery extends BaseQuery
{
}

export interface CatalogArticlesQuery extends BaseQuery
{
    /** The maxium number of results the query should return. */
    maxResults: number;
    /** Id of the catalog. If undefined articles from all available catalogs should be returned. */
    catalogId?: string;
    /** Search string for the query to perfom a full text search. */
    queryString?: string;
    /** If provided, the series id of the returned aticles should match the provided series id. */
    seriesId?: string;
    /** If provided, the base article number of the returned aticles should match the provided base article number. */
    baseArticleNumber?: string;
}

export interface ArticleQuery extends BaseQuery
{
    /** id of the articles catalog */
    catalogId: string;
    /** id of the article */
    articleId: string;
    /** Id of the currently active EAIWS tax scheme */
    taxScheme: string;
}