import { Basket, ItemLockFlags, ItemLockManager, ProjectInfo } from "@egr/wcf/modules/bsk";
import { EaiwsSession } from "@egr/wcf/modules/eaiws";
import { ListenableEvent } from "@egr/wcf/modules/utils";

import { MenuAction } from "../Action";
import { Plugin } from "../Plugin";
import { PluginComponent } from "../PluginComponent";
import { UserRestriction, UserRestrictionsProvider } from "../types/User";
import { ItemList } from "./ItemList";
import { ItemStatusProvider } from "./ItemStatusProvider";
import { ItemTree } from "./ItemTree";
import { ActionProvider } from "./ProjectEditorExtension";
import { ProjectWorkflow } from "./ProjectWorkflow";

/**
 * Mode for the project editor.
 * - `Default`:  Used to create and edit a project.
 * - `Report`:  Used to send/print a report.
 * - `Order`:  Used to place an order.
 */
export type ProjectEditorMode = "Default" | "Report" | "Order";

export interface ProjectMetaData
{
    /** Sales group (SGR) as defined in the OEX specification. */
    salesGroup?: string;
    /** Sales organization (SOR) as defined in the specification. */
    salesOrganization?: string;
    /** Predefined order number of the project */
    orderNumber?: string;
    /** Predefined vendor number of the project */
    vendorNumber?: string;
}

export interface ProjectEditorOptions
{
    /** Html container for the editor ui. */
    container: HTMLElement;

    /** An open eaiws session which will be used from the ui. */
    eaiwsSession: EaiwsSession;

    /** Additional restrictions for the current user. Effective only for this project editor. */
    additionalRestrictions?: ReadonlySet<UserRestriction> | UserRestrictionsProvider;

    /** Additional actions for the editor */
    additionalActions?: ActionProvider;

    /**
     * The editor mode defines the primary purpose of the project editor. If not defined `Default`
     * will be used. The mode may be used by plugins to react to different use cases.
     */
    editorMode?: ProjectEditorMode;

    /**
     * Optional pre-created `ProjectInfo` instance.
     * Will be ignored if `eaiwsSession` and `projectInfo.session` are not equal.
     */
    projectInfo?: ProjectInfo;

    /**
     * If true, legacy data (e.g. header or address data) will be loaded and migrated. (default: false)
     * Should be used when loading obk files of older applications.
     */
    migrateLegacyProjectData?: boolean;

    /**
     * Optional list of locked items (map from basket item id to lock flag)
     * which will be used to lock certain item related operations.
     */
    lockedItems?: ReadonlyMap<string, ItemLockFlags>;

    /**
     * Optional project related meta data which may be used by certain operations (e.g. ordering)
     */
    projectMetaData?: ProjectMetaData;
}

/**
 * The ProjectEditor is the container for all project based plugins like ProjectDataPlugin and ItemListPlugin etc.
 * - Plugin category = `ProjectEditor`
 */
export interface ProjectEditor extends PluginComponent
{
    readonly basket: Basket;
    readonly itemList: ItemList | null;
    readonly itemTree: ItemTree | null;
    readonly itemStatusProvider: ItemStatusProvider | null;
    readonly workflow: ProjectWorkflow | null;

    /** The ItemLockManager is used to lock certain item related operations. */
    readonly itemLockManager: ItemLockManager;

    /**
     * Set of base user restrictions of this project editor.
     * The base restrictions are defined by the application and the host of this project editor.
     * Should not be used directly by most plugins. Use `effectiveRestrictions` instead.
     */
    readonly baseRestrictions: ReadonlySet<UserRestriction>;

    /**
     * Set of currently effective restrictions for the current user.
     * The effective restictions are a combination of the `baseRestrictions` and the current
     * restrictions of the `workflow`. These restrictions may change during life time of this project editor.
     * Plugins should react to the `eventEffectiveRestrictionsChanged` event.
     */
    readonly effectiveRestrictions: ReadonlySet<UserRestriction>;

    /** Current mode of the editor. */
    readonly editorMode: ProjectEditorMode;

    /** If true the editor is used at point of sale. Critical information like discounts should be hidden in this case. */
    readonly pointOfSaleMode: boolean;

    /** Project related meta data which may be used by certain operations (e.g. ordering) */
    readonly projectMetaData: Readonly<ProjectMetaData>;

    /**
    * Will be triggered every time the `baseRestrictions` have changed.
    * Should not be used directly by most plugins. Use `eventEffectiveRestrictionsChanged` instead.
    */
    readonly eventBaseRestrictionsChanged: ListenableEvent<void, ProjectEditor>;

    /**
     * Will be triggered every time the `effectiveRestrictions` have changed.
     */
    readonly eventEffectiveRestrictionsChanged: ListenableEvent<void, ProjectEditor>;

    /**
    * Will be triggered every time the `pointOfSaleMode` has changed.
    */
    readonly eventPointOfSaleModeChanged: ListenableEvent<void, ProjectEditor>;

    /**
    * Will be triggered every time the `projectMetaData` has changed.
    */
    readonly eventProjectMetaDataChanged: ListenableEvent<void, ProjectEditor>;

    /** Will be called everytime the main menu gets opened. */
    getMainMenuActions(): ReadonlyArray<MenuAction> | null;

    /** Used to enable/disable point of sale mode */
    setPointOfSaleMode(pEnabled: boolean): void;

    /** Used to change the project meta data */
    setProjectMetaData(pData: ProjectMetaData): void;

    /** Shows the catalog if not already shown. */
    showCatalog(): void;
}


export interface ProjectEditorPlugin extends Plugin
{
    /** Creates a new ProjectEditor instance. */
    createComponent(pOwner: PluginComponent, pOptions: ProjectEditorOptions): Promise<ProjectEditor>;
}
