import { ListenableEvent } from "@egr/wcf/modules/utils";

import { Plugin } from "../Plugin";
import { ProjectEditor } from "./ProjectEditor";
import { UserRestriction } from "../types/User";
import { Disposable } from "../types/Disposable";

export interface ProjectWorkflowPlugin extends Plugin
{
    createWorkflow(pEditor: ProjectEditor): Promise<ProjectWorkflow>;
}

/**
 * The ProjectWorkflow plugin manages user restrictions and the project state for the current `ProjectEditor`.
 * E.g.: The project state may be changed to `Ordered` when an order is sent.
 * - Plugin category = `ProjectWorkflow`
 */
export interface ProjectWorkflow extends Disposable
{
    /**
     * Set of user resctrictions for the current workflow.
     * Should not be used directly by most plugins. Use `ProjectEditor.effectiveRestrictions` instead.
     */
    readonly workflowRestrictions: ReadonlySet<UserRestriction>;

    /**
     * Will be triggered every time the `workflowRestrictions` have changed.
     * Should not be used directly by most plugins. Use `ProjectEditor.eventEffectiveRestrictionsChanged` instead.
     */
    readonly eventWorkflowRestrictionsChanged: ListenableEvent<void, ProjectWorkflow>;

    /**
     * Returns true if the user is not allowed to manually change the project state.
     * This function has priority over the current user restrictions, and thus can be user to override them.
     */
    isProjectStateReadonly(): Promise<boolean>;

    /**
     * Notifies this workflow, that the project was offered. E.g. can be used to adjust the project state.
     */
    projectOffered(): Promise<void>;

    /**
     * Notifies this workflow, that the project was ordered. E.g. can be used to adjust the project state.
     */
    projectOrdered(): Promise<void>;
}
