import { newError } from "@egr/wcf/modules/utils/error";
import { EaiwsSession } from "@egr/wcf/modules/eaiws";
import * as basket from "@egr/wcf/modules/eaiws/basket";
import { Log } from "@egr/wcf/modules/utils";

import { PluginDescriptor } from "../../plugin-sdk/wbk-core/Plugin";
import { ProjectEditorPlugin, ProjectEditor } from "../../plugin-sdk/wbk-core/plugins/ProjectEditor";
import { MenuAction } from "../../plugin-sdk/wbk-core/Action";
import { ActivityPlugin, ActivityHost, Activity } from "../../plugin-sdk/wbk-core/plugins/Activity";

import wbkApp = egr.wbkApp;

/**
 * Example activity which demonstrates the following:
 * 1. create a new eaiws session
 * 2. create a new user article inside this session
 * 3. open the project editor for this session
 */
class PluginImpl implements ActivityPlugin
{
    public get name(): string
    {
        return ("examplePlugin-activity");
    }


    public initialize(pDescriptor: PluginDescriptor, pSettings: unknown): Promise<void>
    {
        return (Promise.resolve());
    }


    public async createComponent(pHost: ActivityHost): Promise<Activity>
    {
        let tActivity = new ActivityImpl(pHost);
        await tActivity.init();
        return (tActivity);
    }
}


export class ActivityImpl implements Activity
{
    public constructor(pHost: ActivityHost)
    {
        this.mSession = null;
        this.mProjectEditor = null;
        this.mHost = pHost;
    }


    public async init(): Promise<void>
    {
        //create session
        await this.createSession();
        if (this.mSession == null || !this.mSession.isValid) {
            this.mHost.stopActivity(false);
            return;
        }

        //set host title
        this.mHost.title = "Example Activity";
        this.mHost.titleEditable = false;

        //get project editor plugin
        let tDesciptors = wbkApp.pluginManager.getDescriptorsByCategory("ProjectEditor");
        if (tDesciptors.length === 0)
            throw newError("ProjectEditor plugin not found.");
        let tProjectEditorPlugin = await wbkApp.pluginManager.getPlugin(tDesciptors[0].name) as ProjectEditorPlugin;

        //create editor
        this.mProjectEditor = await tProjectEditorPlugin.createComponent(this, {
            container: this.mHost.container,
            eaiwsSession: this.mSession
        });
    }


    public async dispose(): Promise<void>
    {
        if (this.mProjectEditor != null) {
            await this.mProjectEditor.dispose();
            this.mProjectEditor = null;
        }

        if (this.mSession != null) {
            await this.mSession.close();
            this.mSession = null;
        }
    }


    public getMainMenuActions(): ReadonlyArray<MenuAction> | null
    {
        if (this.mProjectEditor != null) {
            return (this.mProjectEditor.getMainMenuActions());
        } else {
            return (null);
        }
    }


    //// private ///////////////////////////////////////////////////////////////////////////////////

    private mSession: EaiwsSession | null;
    private mHost: ActivityHost;
    private mProjectEditor: ProjectEditor | null;


    private async createSession(): Promise<void>
    {
        try {
            this.mSession = await wbkApp.utils.createEaiwsSession();

            //add user article
            await this.createUserArticle(this.mSession);
        } catch (pError: any) {
            Log.error("Failed to start session.", pError);
            wbkApp.showMessageBox(
                {
                    title: wbkApp.commonResources.labelError,
                    message: wbkApp.commonResources.errorStartSessionFailed,
                    detailMessage: pError.toString(),
                    icon: "Error",
                    buttons: []
                });
        }
    }


    private async createUserArticle(pSession: EaiwsSession): Promise<void>
    {
        let tId = await pSession.basket.insertUserArticle(null, null, "My article");

        let tItemProps = new basket.ItemProperties();
        tItemProps.article = new basket.ArticleProperties();
        tItemProps.article.manufacturerId = "XYZ";
        tItemProps.article.seriesId = "ABC";
        tItemProps.article.baseArticleNumber = "12345";
        tItemProps.article.currency = "EUR";
        tItemProps.article.salesPrice = 100;
        tItemProps.article.purchasePrice = 90;
        tItemProps.article.longText = "description of my article";

        await pSession.basket.setItemProperties(tId, tItemProps);
    }
}


//register plugin
wbkApp.pluginManager.registerPlugin(new PluginImpl());