import { ListenableEvent } from "@egr/wcf/modules/utils";
import { ArticleItem, BasketItem, ItemUpdateFlags, SetArticleItem } from "@egr/wcf/modules/bsk";
import { delay } from "@egr/wcf/modules/utils/async";

import { PluginDescriptor } from '../../plugin-sdk/wbk-core/Plugin';
import { ItemDescriptionProviderPlugin, ItemDescriptionProvider, ItemDescriptionLine } from '../../plugin-sdk/wbk-core/plugins/ItemListExtension';
import { ProjectEditor } from '../../plugin-sdk/wbk-core/plugins/ProjectEditor';

import wbkApp = egr.wbkApp;

/** implementation of ItemDescriptionProvider plugin interface */
class PluginImpl implements ItemDescriptionProviderPlugin
{
    public constructor(pIsProvider2: boolean)
    {
        this.mIsProvider2 = pIsProvider2;
    }

    public get name(): string
    {
        return (this.mIsProvider2 ? "examplePlugin-itemDescriptionProvider2" : "examplePlugin-itemDescriptionProvider1");
    }

    public async initialize(pDescriptor: PluginDescriptor, pSettings: unknown): Promise<void>
    {
    }

    public async createProvider(pEditor: ProjectEditor): Promise<ItemDescriptionProvider>
    {
        //return new insance of our provider implementation
        return (new Provider(pEditor, this.mIsProvider2));
    }

    private mIsProvider2: boolean;
}


/**
 * implementation of ItemDescriptionProvider interface
 */
class Provider implements ItemDescriptionProvider
{
    readonly eventItemDescriptionChanged: ListenableEvent<BasketItem, ItemDescriptionProvider> = new ListenableEvent<BasketItem, ItemDescriptionProvider>(this);

    public constructor(pEditor: ProjectEditor, pIsProvider2: boolean)
    {
        this.mEditor = pEditor;
        this.mIsProvider2 = pIsProvider2;
        this.mEditor.basket.eventItemUpdated.addListener(this.onItemUpdated);
        this.mDescriptions = new Map();
    }

    public async fetchItemDescriptions(pItems: ReadonlyArray<BasketItem>): Promise<void>
    {
        //simulate slow request
        await delay(Math.random() * 5000);

        for (let tItem of pItems) {
            if (tItem instanceof ArticleItem && !(tItem instanceof SetArticleItem)) {
                if (this.mIsProvider2) {
                    this.mDescriptions.set(tItem.id, {
                        text: "My important information",
                        color: "#ff0000"
                    });
                } else {
                    this.mDescriptions.set(tItem.id, {
                        text: "My custom description"
                    });
                }
            } else {
                this.mDescriptions.set(tItem.id, null);
            }
        }
    }

    public getItemDescription(pItem: BasketItem): ItemDescriptionLine | null | undefined
    {
        return (this.mDescriptions.get(pItem.id));
    }

    public async dispose(): Promise<void>
    {
        this.mEditor.basket.eventItemUpdated.removeListener(this.onItemUpdated);
    }

    private onItemUpdated = (pParam: { item: BasketItem; flags: ItemUpdateFlags; }): void =>
    {
        //remove cached descriptions for this item
        this.mDescriptions.delete(pParam.item.id);

        //notify listeners that status has changed
        this.eventItemDescriptionChanged.trigger(pParam.item);
    }


    private mEditor: ProjectEditor;
    private mIsProvider2: boolean;
    private mDescriptions: Map<string, ItemDescriptionLine | null>; //map from basket item id to description
}


//register plugin
wbkApp.pluginManager.registerPlugin(new PluginImpl(false));
wbkApp.pluginManager.registerPlugin(new PluginImpl(true));