import { ListenableEvent } from "@egr/wcf/modules/utils";
import { ArticleItem, BasketItem, ItemUpdateFlags, SetArticleItem } from "@egr/wcf/modules/bsk";

import { PluginDescriptor } from '../../plugin-sdk/wbk-core/Plugin';
import { ItemStatusProviderPlugin, ItemStatusProvider, ItemStatusIcon } from '../../plugin-sdk/wbk-core/plugins/ItemStatusProvider';
import { ProjectEditor } from '../../plugin-sdk/wbk-core/plugins/ProjectEditor';

import wbkApp = egr.wbkApp;


/** implementation of ItemStatusProviderPlugin plugin interface */
class PluginImpl implements ItemStatusProviderPlugin
{
    public get name(): string
    {
        return ("examplePlugin-itemStatusProvider");
    }

    public async initialize(pDescriptor: PluginDescriptor, pSettings: unknown): Promise<void>
    {
    }

    public async createProvider(pEditor: ProjectEditor): Promise<ItemStatusProvider>
    {
        //return new insance of our provider implementation
        return (new Provider(pEditor));
    }
}


/**
 * implementation of ItemStatusProvider interface which generates a status based on the sales price
 */
class Provider implements ItemStatusProvider
{
    readonly eventItemStatusChanged: ListenableEvent<BasketItem, ItemStatusProvider> = new ListenableEvent<BasketItem, ItemStatusProvider>(this);

    public constructor(pEditor: ProjectEditor)
    {
        this.mEditor = pEditor;
        this.mEditor.basket.eventItemUpdated.addListener(this.onItemUpdated);
        this.mItemStatus = new Map();
    }

    public async fetchItemStatus(pItems: ReadonlyArray<BasketItem>): Promise<void>
    {
        let tItemIdsToFetch = new Set(pItems.map((pItem) => pItem.id));

        //fetch all item properties at once for better performance
        await this.mEditor.basket.root.fetchItemProperties(true, (pItem) => tItemIdsToFetch.has(pItem.id));

        for (let tItem of pItems) {
            //use cached item properties, should be available after fetch
            let tProps = tItem.getCachedItemProperties();
            if (tProps != null && tProps.article != null && tProps.article.salesPrice != null) {
                //generate example status based on sales price
                if (tProps.article.salesPrice < 500)
                    this.mItemStatus.set(tItem.id, "green");
                else if (tProps.article.salesPrice > 1000)
                    this.mItemStatus.set(tItem.id, "red");
                else
                    this.mItemStatus.set(tItem.id, "yellow");
            } else {
                this.mItemStatus.set(tItem.id, null);
            }
        }
    }

    public getItemStatusIcon(pItem: BasketItem): ItemStatusIcon | null | undefined
    {
        if (pItem instanceof ArticleItem && !(pItem instanceof SetArticleItem)) {
            let tStatus = this.mItemStatus.get(pItem.id);
            if (tStatus === undefined)
                return (undefined); //status is unknown

            switch (tStatus)
            {
                case "red":
                    return {
                        iconName: "mdi-emoticon-sad-outline",
                        iconColor: "#FF0000",
                        tooltip: "high price",
                        onClick: () => alert("high price")
                    };

                case "yellow":
                    return {
                        iconName: "mdi-emoticon-neutral-outline",
                        iconColor: "#ffb300",
                        tooltip: "medium price",
                        onClick: () => alert("medium price")
                    };

                case "green":
                    return {
                        iconName: "mdi-emoticon-happy-outline",
                        iconColor: "#00cc00",
                        tooltip: "low price",
                        onClick: () => alert("low price")
                    };
                default:
                    return (null);
            }

        } else {
            //no icon for folders ,text and set items
            return (null);
        }
    }

    public async dispose(): Promise<void>
    {
        this.mEditor.basket.eventItemUpdated.removeListener(this.onItemUpdated);
    }

    private onItemUpdated = (pParam: { item: BasketItem; flags: ItemUpdateFlags; }): void =>
    {
        //remove cached status for this item
        this.mItemStatus.delete(pParam.item.id);

        //notify listeners that status has changed
        this.eventItemStatusChanged.trigger(pParam.item);
    }


    private mEditor: ProjectEditor;
    private mItemStatus: Map<string, "red" | "yellow" | "green" | null>; //map from basket item id to status
}


//register plugin
wbkApp.pluginManager.registerPlugin(new PluginImpl());