import { ListenableEvent } from "@egr/wcf/modules/utils";
import { LengthUnit } from "@egr/wcf/modules/utils/number";
import { ProjectState } from "@egr/wcf/modules/eaiws/project";

import { UserRestriction } from "./types/User";

export interface EaiwsSettings
{
    /** base url of the EAIWS server */
    baseUrl: string | null;

    /** startup to use for session creation */
    startup: string | null;

    /**
     * Interval for the session keep alive message in milliseconds. (default: 60000, 0 disables keep alive)
     * Only used if session is created with `baseUrl` and `startup`.
     */
    keepAliveInterval?: number;

    /**
     * gatekeeper ID to use for session creation,
     * ignored if `baseUrl` and `startup` are defined
     */
    gatekeeperId: string | null;

    /**
     * Application ID used to connect to gatekeeper.
     */
    gatekeeperAppId?: string;
}

export interface CalculationSettings
{
    /** name of the pricing procedure to use for the project and item calculation */
    pricingProcedure: string;
}

export interface ReportAttachments {
    /** List of PDF urls which should be added in front of the report. */
    prependUrls?: Array<string>;
    /** List of PDF urls which should be added at the end of the report. */
    appendUrls?: Array<string>;
}

export interface ReportSettings
{
    /**
     * Array of template names of report templates which should be available to the user.
     * If null all available templates will be shown.
     */
    templates: Array<string> | null;

    /** Name of the default template */
    defaultTemplate: string | null;

    /**
     * Predefined attachments for each template which will be added to every report.
     * The name of the template is used as the key for the dictionary.
     * If an empty string is used as the template name the provided attachments
     * will be applied to all templates without explicitly defined attachments.
     */
    attachments?: { readonly [template: string]: ReportAttachments };
}

export interface OrderSettings
{
    /**
     * If defined a report using the specified template will
     * be generated and attached to the order. (default: null)
     */
    reportTemplate: string | null;
}

export type DeleteItemConfirmation = "Always" | "MultipleItems" | "Never";

export interface ProjectEditorSettings
{
    /** Controls if the delete confirmation dialog should be shown. (default: "Always") */
    deleteItemConfirmation: DeleteItemConfirmation;

    /** Controls if the catalog should be opened automatically for empty/new projects. (default: false) */
    openCatalogForEmptyProjects: boolean;
}

export interface WebGLSettings
{
    /** Force the use of WebGL 1 instad of WebGL 2. (default: false) */
    forceWebGL1: boolean;

    /** Enabled realtime shadows. (default: true) */
    shadowsEnabled: boolean;

    /**
     * Special mode for devices without GPU (no hardware acceleration). (default: false)
     * @experimental May be removed or changed in future.
     */
    noGPU?: boolean;
}

/** default application settings */
export interface DefaultSettings
{
    dataLanguage: ReadonlyArray<string>;
    calculation: Readonly<CalculationSettings>;
    eaiws: Readonly<EaiwsSettings>;
    userRestrictions: ReadonlySet<UserRestriction>;
    theme: Readonly<ThemeSettings>;
    report: Readonly<ReportSettings>;
    order: Readonly<OrderSettings>;
    projectEditor: Readonly<ProjectEditorSettings>;
}


export interface ThemeSettings
{
    /** Primary color of the theme as hex string. (default: #0078d4) */
    primaryColor: string;

    /** Text color of the theme as hex string. (default: #333333) */
    textColor: string;

    /** Application bar color of the theme as hex string. (default: #ffffff) */
    appBarColor: string;

    /**
     * Logo of the application. (default: null)
     */
    logo: string | null;

    /**
     * Background color of the logo as hex/css color string. (default: transparent)
     */
    logoBackgroundColor: string;
}

export interface ApplicationSettings
{
    /** event which triggers if the locale is changed */
    readonly eventLocaleChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the length unit is changed */
    readonly eventLengthUnitChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the data language is changed */
    readonly eventDataLanguageChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the project validity is changed */
    readonly eventProjectValidityChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the calculation settings are changed */
    readonly eventCalculationChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the eaiws settings are changed */
    readonly eventEaiwsChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the theme settings are changed */
    readonly eventThemeChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the report settings are changed */
    readonly eventReportChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the order settings are changed */
    readonly eventOrderChanged: ListenableEvent<void, ApplicationSettings>;

    /** event which triggers if the project editor settings are changed */
    readonly eventProjectEditorChanged: ListenableEvent<void, ApplicationSettings>;

    /** UI language of the application (based on the current locale) */
    readonly language: string;

    /**
     * Default language used for the product data (Array of languages in order of priority).
     * If the array is empty the UI language will be used as fallback.
     * Use `EaiwsSessionDefaults.applyDefaults()` to apply the default language.
     */
    readonly dataLanguage: ReadonlyArray<string>;

    /**
     * Default project validity. The validity is represented as an ISO 8601 duration string.
     * An empty string is used for `no validity`.
     * Use `EaiwsSessionDefaults.applyDefaults()` to apply the default project validity.
     */
    readonly projectValidity: string;

    /**
     * ID of the special model information text.
     */
    readonly specialModelInfoTextId: string;

    /**
     * True if the articles price date is supported.
     */
    readonly priceDateSupported: boolean;

    /**
     * Locale of the application. Used for number and date formatting and
     * defines the language of the application.
     */
    readonly locale: string;

    /** Length unit which is used to show length values. */
    readonly lengthUnit: LengthUnit;

    /** List of supported project states in correct order. */
    readonly supportedProjectStates: ReadonlyArray<ProjectState>;

    /** current calculation settings */
    readonly calculation: Readonly<CalculationSettings>;

    /** current eaiws settings */
    readonly eaiws: Readonly<EaiwsSettings>;

    /** current theme settings */
    readonly theme: Readonly<ThemeSettings>;

    /** current report settings */
    readonly report: Readonly<ReportSettings>;

    /** current order settings */
    readonly order: Readonly<OrderSettings>;

    /** current project editor settings */
    readonly projectEditor: Readonly<ProjectEditorSettings>;

    /** current WebGL settings */
    readonly webGL: Readonly<WebGLSettings>;

    /** default application settings */
    readonly defaults: Readonly<DefaultSettings>;

    /**
     * Settings storage which stores settings for the current browser.
     * Note: This is a synchronous api.
     */
    readonly browserStorage: SettingsStorage;

    /**
     * Cross browser settings storage which stores settings for the current user.
     * Only available if the user authentication provides a storage.
     * Note: This is an asynchronous api.
     */
    readonly userStorage: AsyncSettingsStorage | null;

    /** Sets new data language. */
    setDataLanguage(pLanguage: ReadonlyArray<string>): void;

    /** Sets new project validity. */
    setProjectValidity(pValidity: string): void;

    /** Sets new calculation settings. */
    setCalculation(pCalculation: CalculationSettings): void;

    /** Sets new eaiws settings. */
    setEaiws(pEaiws: EaiwsSettings): void;

    /** Sets new theme settings. */
    setTheme(pTheme: ThemeSettings): void;

    /** Sets new report settings. */
    setReport(pReport: ReportSettings): void;

    /** Sets new order settings. */
    setOrder(pOrder: OrderSettings): void;

    /** Sets new project editor settings. */
    setProjectEditor(pProjectEditor: ProjectEditorSettings): void;

    /** Enables/disables article price date support. */
    setPriceDateSupported(pSupported: boolean): void;
}

/** Synchronous settings storage. */
export interface SettingsStorage
{
    /**
     * Sets or overrides the persisted value of a setting.
     * Values have to be serializable as a JSON string.
     * The group key should be a unique identifer (e.g. a plugin name or vendor)
     * `undefined` can be used as value to delete the setting the setting from the storage.
     */
    setSettingJson(pGroupKey: string, pKey: string, pValue: unknown): void;

    /**
     * Retrieves the persisted value of a setting. The value will be parsed as a JSON string.
     * Undefined will be returend if the setting does currently not have a value.
     */
    getSettingJson<T>(pGroupKey: string, pKey: string): T | undefined;

    /**
     * Sets or overrides the persisted string value of a setting.
     * The group key should be a unique identifer (e.g. a plugin name or vendor)
     * `undefined` can be used as value to delete the setting from the storage.
     */
    setSettingString(pGroupKey: string, pKey: string, pValue: string | undefined): void;

    /**
     * Retrieves the persisted string value of a setting. `undefined` will be returend if the setting
     * does not have a value yet.
     */
    getSettingString(pGroupKey: string, pKey: string): string | undefined;
}

/** Asynchronous settings storage. */
export interface AsyncSettingsStorage
{
    /**
     * Sets or overrides the persisted value of a setting.
     * Values have to be serializable as a JSON string.
     * The group key should be a unique identifer (e.g. a plugin name or vendor)
     * `undefined` can be used as value to delete the setting the setting from the storage.
     */
    setSettingJson(pGroupKey: string, pKey: string, pValue: unknown): Promise<void>;

    /**
     * Retrieves the persisted value of a setting. The value will be parsed as a JSON string.
     * Undefined will be returend if the setting does currently not have a value.
     */
    getSettingJson<T>(pGroupKey: string, pKey: string): Promise<T | undefined>;

    /**
     * Sets or overrides the persisted string value of a setting.
     * The group key should be a unique identifer (e.g. a plugin name or vendor)
     * `undefined` can be used as value to delete the setting from the storage.
     */
    setSettingString(pGroupKey: string, pKey: string, pValue: string | undefined): Promise<void>;

    /**
     * Retrieves the persisted string value of a setting. `undefined` will be returend if the setting
     * does not have a value yet.
     */
    getSettingString(pGroupKey: string, pKey: string): Promise<string | undefined>;
}

