import { Basket, ProjectInfo } from "@egr/wcf/modules/bsk";
import { EaiwsSession } from "@egr/wcf/modules/eaiws";
import * as project from "@egr/wcf/modules/eaiws/project";
import * as session from "@egr/wcf/modules/eaiws/session";

import { i18n } from "i18next";
import { EaiwsSettings } from "./ApplicationSettings";
import { UserRestriction } from "./types/User";

export type ImageMimeType = "image/png" | "image/jpeg";

export interface CreateBasketOptions
{
    /** Optional pre - created `ProjectInfo` instance. Will be ignored if basket session and `projectInfo.session` are not equal. */
    projectInfo?: ProjectInfo;

    /**
     * If true, legacy data (e.g. header or address data) will be loaded and migrated. (default: false)
     * Should be used when loading obk files of older applications.
     */
    migrateLegacyProjectData?: boolean;
}

export interface CreateEaiwsSessionOptions
{
    /**
     * Can be used to create a custom session.
     * If provided the application settings will be ignored.
     */
    customSettings?: EaiwsSettings;

    /**
     * pCon.login token to use for session creation.
     * The login token is only relevant if the Gatekeeper is used.
     */
    pConLoginToken?: string;

    /**
     * Eaiws authentication data to use for session creation.
     * The authentication data will be ignored if the Gatekeeper is used.
     */
    eaiwsAuthData?: EaiwsAuthData;

    /**
     * If true and `wbkApp.user.eaiwsSession` is available this session will be used and it will be cleared and prepared. (default: true)
     * No new session will be created in this case. All other options will be ignored.
     * If `wbkApp.user.eaiwsSession` is not available a new session will be created using the provided options.
     * Note: `wbkApp.user.eaiwsSession` should not be closed. Use `Utils.closeEaiwsSession()` to ensure this.
     */
    useUserSession?: boolean;

    /**
     * If true the session will be closed automatically if the browser window gets closed. (default: true)
     */
    closeSessionOnWindowClose?: boolean;
}

export interface UploadToEaiwsSessionOptions
{
    /** eaiws session to use for upload */
    session: EaiwsSession;
    /** eaiws file type of the uploaded file */
    fileType: session.SessionCacheFileType;
    /** data of the uploaded file */
    fileData: Blob | File;
    /** suffix which will be added to the upload url e.g. a file extension */
    urlSuffix?: string;
    /** if false the progress will not be shown (default: true) */
    showProgress?: boolean;
    /** optional custom title for the upload progress */
    progressTitle?: string;
    /** optional custom progress handler */
    onProgress?: (pProgress: number | null, pAbortHandler: () => void) => void;
}

export interface ResizeImageOptions
{
    /** data of the image */
    imageData: Blob | File;
    /** mimeType of the resized image (default: "image/png") */
    mimeType?: ImageMimeType;
    /** maximum pixel size of the image (default: 1024) */
    maxSize?: number;
}

export interface UploadImageToEaiwsSessionOptions extends Pick<UploadToEaiwsSessionOptions & ResizeImageOptions,
    "session" | "showProgress" | "progressTitle" | "onProgress" | "maxSize">
{
    /**
     * Image as file, url or data URI.
     * If `imageDataMimeType` is undefined and the given url does not constain a file extension the image will be handled as a JPG.
     */
    imageData: File | Blob | string;

    /**
     * Optional mimeType of the image data. If undefined `uploadImageToEaiwsSession` is trying to detect the mimeType.
     */
    imageDataMimeType?: ImageMimeType;
}

export interface UploadToSharedFileStorageOptions
{
    /** Data of the file. If a string is used the string has to contain a valid download url for the file. */
    fileData: Blob | File | string;
    /** Name of the file including file extension */
    fileName: string;
}

export interface EaiwsAuthData
{
    /** user token of the authorized user */
    userToken: string;

    /** the data domain for the authorized user */
    dataDomain?: string;
}

/** Will be thrown from `Utils.createEaiwsSession()` in case of gatekeeper errors. */
export type GatekeeperError = Error & {
    gatekeeperErrorDetails?: GatekeeperErrorDetails;
};

export interface GatekeeperErrorDetails
{
    id: string;
    type: string;
    code: number;
    scopes: Array<string>;
    message: string;
    info?: GatekeeperErrorInfo;
}

export interface GatekeeperErrorInfo
{
    name?: string;
    stack?: string;
    message?: string;
}

export interface SharedFile
{
    url: string;
    expireDate: string;
}

export interface Utils
{
    getBrowserLocale(): string;
    initializeI18n(pLanguage: string, pResourceVersion: string, pResourcesPath: string): Promise<i18n>;

    /**
     * Creates and prepares a new eaiws session using the current application settings.
     * In case of a gatekeeper error a `GatekeeperError` will be thrown.
     * Note: `prepareEaiwsSession()` will be also called internally.
     */
    createEaiwsSession(pOptions?: CreateEaiwsSessionOptions): Promise<EaiwsSession>;

    /**
     * Should be called after starting a session to setup parameters like data language and locale of the session.
     * Note: Will be called automatically from `createEaiwsSession()`
     */
    prepareEaiwsSession(pSession: EaiwsSession): Promise<void>;

    /**
     * This function will close the given eaiws session. If the given session is equal to `wbkApp.user.eaiwsSession`
     * than `EaiwsSession.session.loadEmptySession()` will be used instead to clear the session.
     * @param pForceClose If true the session will be closed even if it is the user session.
     */
    closeEaiwsSession(pSession: EaiwsSession, pForceClose?: boolean): Promise<void>;

    /**
     * Uploads a file to an eaiws session while showing a progress. The url of the uploaded file will be returned.
     * `null` will be returned if the upload was aborted by the user.
     */
    uploadToEaiwsSession(pOptions: UploadToEaiwsSessionOptions): Promise<string | null>;

    /**
    * Uploads and resizes an image to an eaiws session while showing a progress. The url of the uploaded image will be returned.
    * `null` will be returned if the upload was aborted by the user.
    * Only PNG and JPG images are supported.
    */
    uploadImageToEaiwsSession(pOptions: UploadImageToEaiwsSessionOptions): Promise<string | null>;

    /**
     * Uploads the given file to the shared file storage and returns its download url (share link).
     */
    uploadToSharedFileStorage(pOptions: UploadToSharedFileStorageOptions): Promise<SharedFile>;

    /**
     * Returns a list of project groups available for the given session.
     * Note: The server response will be cached to speed up further calls.
     */
    getEaiwsProjectGroups(pSession: EaiwsSession): Promise<Array<project.ProjectGroup>>;

    /**
     * If the project groups have changed this function should be called to clear the internal cache.
     * see `getEaiwsProjectGroups`
     */
    clearEaiwsProjectGroupsCache(pSession: EaiwsSession): void;

    /**
    * Returns a list of price lists available for the given session.
    * Note: The server response will be cached to speed up further calls.
    */
    getEaiwsPriceLists(pSession: EaiwsSession): Promise<Array<project.PriceList>>;

    /**
     * If the price lists have changed this function should be called to clear the internal cache.
     * see `getEaiwsPriceLists`
     */
    clearEaiwsPriceListsCache(pSession: EaiwsSession): void;

    /**
     * Creates and initializes a new basket instance.
     * @param pSession Session the basket should operate on.
     * @param pOptions Options for the created basket.
     */
    createBasket(pSession: EaiwsSession, pOptions?: CreateBasketOptions): Promise<Basket>;

    isInputAreaFocused(): boolean;

    isApplePlatform(): boolean;

    /**
    * Returns true if the ctrl key is currenty down. For Apple platforms the command (meta) key is checked instead.
    */
    isCtrlKeyDown(pEvent: KeyboardEvent | MouseEvent): boolean;

    /**
     * Scrolls an html element into view if necessary.
     */
    scrollElementIntoView(pElement: HTMLElement): void;

    /**
     * Normalizes the given set of user restrictions. E.g. if `edit` is part of the set, all the other
     * editing restrictions will be added also to the set.
     */
    normalizeUserRestrictions(pRestrictions: Set<UserRestriction>): void;

    /**
     * Resizes an image if it exeeds the maximum size. The original image will be returned in case it does not need to be resized.
     */
    resizeImage(pOptions: ResizeImageOptions): Promise<Blob>;

    /**
     * Starts the download of the given file while preventing the `window.beforeunload` event to be triggered.
     * Note: This is not guaranteed to work for cross domain urls (e.g. for PDFs). In this case the file will be opened in a new tab.
     */
    startFileDownload(pFileOrUrl: File | Blob | string, pDownloadFilename?: string): void;

    /**
     * Returns a file name for the given project and extension.
     * @param pBasket Basket instance with loaded project
     * @param pFileExtension Extension of the file (e.g. ".obk")
     */
    getProjectFileName(pBasket: Basket, pFileExtension: string): Promise<string>;

    /**
     * Can be used to programmatically open a link.
     * @param pTarget Custom target for the link (default: `_blank`)
     */
    openLink(pUrl: string, pTarget?: string): void;

    /**
     * Returns a download link for the given url using a CORS proxy.
     * This can be used to prevent CORS errors if the server does not support CORS.
     */
    getCorsProxyUrl(pUrl: string): string;
}