import { BasketItem } from "@egr/wcf/modules/bsk";
import * as basket from "@egr/wcf/modules/eaiws/basket";
import * as catalog from "@egr/wcf/modules/eaiws/catalog";

import { PluginComponent } from "../PluginComponent";
import { BaseComponentExtension } from "./ProjectEditorExtension";

export interface Money
{
    value: number;
    currency: string;
}

export interface Value
{
    value: number;
    unit: string;
}

export interface PackagingInfo
{
    width?: Value;
    height?: Value;
    depth?: Value;
    volume?: Value;
    tareWeight?: Value;
    netWeight?: Value;
    itemsPerPackUnit?: number;
    packUnitsPerArticle?: number;
}

export interface CustomArticle
{
    manufacturerId?: string;
    seriesId?: string;
    baseArticleNumber?: string;
    finalArticleNumber?: string;
    variantCode?: string;

    /**
     * Quantity of the article
     * @defaultValue 1
     */
    quantity?: number;

    /** short text as simple string or as multi language map (language code -> text) */
    shortText?: string | Map<string, string> | { [language: string]: string };
    /** long text as simple string or as multi language map (language code -> text) */
    longText?: string | Map<string, string> | { [language: string]: string };
    /** feature text as simple string or as multi language map (language code -> text) */
    featureText?: string | Map<string, string> | { [language: string]: string };

    /** Additional comments/notes for this article. */
    comments?: string;

    purchasePrice?: Money;
    salesPrice?: Money;
    packagingInfo?: PackagingInfo;

    /** Map from tax type to tax category for the active tax scheme. */
    taxes?: Map<string, string> | { [type: string]: string };

    /**
     * Url or data-URI for the article image.
     * Currently only jpg and png images are supported.
     */
    imageUrl?: string;
}

export interface CatalogDescription {
    id: string;
    label: string;
    imageUrl?: string;
}

export interface ArticleSearchParams
{
    /* Id of the catalog. Returned from `getAvailableCatalogs()`. */
    catalogId: string;
    baseArticleNumber: string;
    seriesId?: string;
}

export interface MutipleArticleOptions
{
    articleOptions: Array<ArticleOption>;
    /** header text for the list of article options */
    headerText?: string;
    /** Insert the article/option with given id */
    insertOption(pOptionId: string): Promise<void>;
}

export interface ArticleOption
{
    id: string;
    label: string;
}

export interface Catalog extends PluginComponent
{
    onInsertArticle: ((pArticle: catalog.ArticleCatalogItem | basket.InsertInfo) => Promise<void>) | undefined;
    onInsertContainer: ((pCatalogItem: catalog.CatalogItem) => Promise<void>) | undefined;
    onInsertCustomArticle: ((pArticle: CustomArticle, pUpdateItem?: BasketItem) => Promise<void>) | undefined;

    /** Returns a list of available catalogs */
    getAvailableCatalogs(): Promise<Array<CatalogDescription>>;

    /** Returns a list of suggested article numbers when a partial baseArticleNumber is given. */
    getArticleNumberSuggestions(pParams: ArticleSearchParams): Promise<Array<string>>;

    /**
     * Searches for an article and inserts it.
     * Return values:
     *  - `false`: no article was found
     *  - `true`: article was found and inserted
     *  - `MutipleArticleOptions`: multiple options were found
     */
    searchAndInsertArticle(pParams: ArticleSearchParams): Promise<boolean | MutipleArticleOptions>;
}

/**
 * Plugin which is used to browse the OFML catalog and insert articles.
 * - Plugin category = `Catalog`
 */
export interface CatalogPlugin extends BaseComponentExtension<Catalog>
{
}
